<?php

namespace Teknisa\MAN\Services;

use Teknisa\MAN\Utilities;

class RelMetaVersusVendasService {
	
	protected $entityManager;

	public function __construct(\Doctrine\ORM\EntityManager $entityManager)  {
		$this->entityManager = $entityManager;
	}
	
	public function fetchReport($conditions){
	    $params = self::buildQueryParams($conditions);
	    $result = [];
	    try{
	        if($params[':CDFILIAL'] != '' || $params[':MES'] != '' || $params[':ANO'] != '') {
	            $result = $this->entityManager->getConnection()->fetchAll(self::VENDAS_VERSUS_META_QRY, $params);
                $result = self::operateParameters($result, $params);  
	        }
            return $result;
	    } catch (\Exception $e){
            throw $e;
        }
	}
	
	
    private function buildQueryParams($conditions){
        if(array_key_exists('CDFILIAL', $conditions)){
            $conditions['CDFILIAL'] = implode('_', $conditions['CDFILIAL']);
        } else {
            $conditions['CDFILIAL'] = '';
        }
        
        if(array_key_exists('MESDATA', $conditions)){
            $conditions['MESDATA'] = implode('_', $conditions['MESDATA']);
        } else {
            $conditions['MESDATA'] = '';
        }
        
        if(!array_key_exists('ANODATA', $conditions)) {
            $conditions['ANODATA'] = '';
        }

        $params = array(
          ':NRORG'    => $conditions['NRORG'],
          ':CDFILIAL' => $conditions['CDFILIAL'],
          ':MES'      => $conditions['MESDATA'],
          ':ANO'      => $conditions['ANODATA'],
        );
        return $params;
    }
  
    private function operateParameters($result, $params){
	    foreach($result as &$rows){
	        $params['CDFILIAL'] = $rows['CDFILIAL'];
	        $rows['ANOMES']     = $rows['NMMES'].' de '.$rows['CDANO'];
 	        $rows['PERCENTUAL'] = floatval($rows['PERCENTUAL']);
	    }
	    
	    return $result;
	}
	
	const VENDAS_VERSUS_META_QRY = "
        SELECT A.*,
             (A.VALOR_FATURAMENTO - A.VRDESITVEND + A.VRACRITVEND) AS TOTAL_FATURAMENTO,
            ((A.VALOR_FATURAMENTO - A.VRDESITVEND + A.VRACRITVEND) - A.VRMES) AS VRMENOSMETA,
            TRUNC(((A.VALOR_FATURAMENTO - A.VRDESITVEND + A.VRACRITVEND)/(A.VRMES)*100),2) AS PERCENTUAL,
            TRUNC(((A.VALOR_FATURAMENTO - A.VRDESITVEND + A.VRACRITVEND)/(A.NRPESMESAVENDA)),2) AS TCKMED
        FROM 
          (SELECT FILOCAVE.CDFILIAL, FILOCAVE.NRORG, FILOCAVE.NMFILIAL,
          FILOCAVE.NRPESMESAVENDA AS NRPESMESAVENDA, FILOCAVE.VRMES, TO_CHAR(DTENTRVENDA, 'MM') AS MES, TO_CHAR(DTENTRVENDA, 'YYYY') AS ANO,
          CASE FILOCAVE.CDMES 
            WHEN 1 THEN 'Janeiro'
            WHEN 2 THEN 'Fevereiro'
            WHEN 3 THEN 'Março'
            WHEN 4 THEN 'Abril'
            WHEN 5 THEN 'Maio'
            WHEN 6 THEN 'Junho'
            WHEN 7 THEN 'Julho'
            WHEN 8 THEN 'Agosto'
            WHEN 9 THEN 'Setembro'
            WHEN 10 THEN 'Outubro'
            WHEN 11 THEN 'Novembro'
            WHEN 12 THEN 'Dezembro'
          END AS NMMES, FILOCAVE.CDANO, SUM(NVL(I.VRDESITVEND, 0)) AS VRDESITVEND, 
          CASE FILOCAVE.IDSITUVENDA
              WHEN 'C' THEN 0
              ELSE SUM(TRUNC(NVL(I.VRACRITVEND, 0), 2))
          END AS VRACRITVEND,
          CASE FILOCAVE.IDSITUVENDA
              WHEN 'C' THEN 0
              ELSE SUM(TRUNC(I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL, 0)), 2))
          END AS VALOR_FATURAMENTO
        FROM ITEMVENDA I
         JOIN (
        	SELECT V.IDSITUVENDA, V.IDORIGEMVENDA, V.NRSEQVENDA, V.DTVENDA, V.DTENTRVENDA, FILOCA.CDCAIXA, FILOCA.NMCAIXA, FILOCA.CDLOJA, FILOCA.NMLOJA, FILOCA.CDFILIAL,
                            FILOCA.NMFILIAL, NVL(V.VRREPIQUEVENDA, 0) AS VRREPIQUEVENDA, V.DTABERVENDA,
                            FILOCA.NRORG, SUM(NVL(V.NRPESMESAVENDA, 1)) OVER (PARTITION BY FILOCA.CDFILIAL,FILOCA.CDMES, FILOCA.CDANO) AS NRPESMESAVENDA, FILOCA.VRMES, FILOCA.CDMES, FILOCA.CDANO,
                            FILOCA.CDPRODTAXAENTR, FILOCA.IDTRATTAXAENTR, FILOCA.CDPRODTAXASERV, FILOCA.IDTRATTAXASERV, FILOCA.IDCOMISVENDA
                        FROM VENDA V
                        JOIN (SELECT C.CDCAIXA, C.NMCAIXA, FILO.CDLOJA, FILO.NMLOJA, FILO.CDFILIAL, FILO.NMFILIAL, FILO.NRORG, FILO.CDMES, FILO.CDANO, FILO.VRMES,
                              FILO.CDPRODTAXAENTR, FILO.IDTRATTAXAENTR, FILO.CDPRODTAXASERV, FILO.IDTRATTAXASERV, FILO.IDCOMISVENDA
                                 FROM CAIXA C
                                 JOIN (SELECT L.CDLOJA, L.NMLOJA, L.IDCOMISVENDA, L.CDPRODTAXAENTR, L.IDTRATTAXAENTR, L.CDPRODTAXASERV, L.IDTRATTAXASERV, FIPA.CDFILIAL, FIPA.NMFILIAL, FIPA.NRORG, FIPA.CDMES, FIPA.CDANO, FIPA.VRMES
                                         FROM LOJA L
                                         JOIN (SELECT FI.CDFILIAL, FI.NMFILIAL, FI.NRORG,  CASE ME.VRMES WHEN 0 THEN 1 ELSE ME.VRMES END AS VRMES, ME.CDMES, ME.CDANO, ME.IDATIVO
                                                FROM
                                                  METAMES ME                                                  
                                                 JOIN FILIAL FI
                                                  ON ME.CDFILIAL = FI.CDFILIAL                                                 
                                                WHERE INSTR(:CDFILIAL, FI.CDFILIAL) > 0 
                                                  AND ME.IDATIVO = 'S'
                                                  AND FI.NRORG = :NRORG
                                                  AND INSTR(:MES, ME.CDMES) > 0
                                                  AND ME.CDANO = :ANO
                                              ) FIPA
                                           ON L.CDFILIAL = FIPA.CDFILIAL
                                            AND L.NRORG = FIPA.NRORG) FILO 
                                   ON C.CDFILIAL = FILO.CDFILIAL 
                                    AND C.CDLOJA = FILO.CDLOJA
                                    AND C.NRORG = FILO.NRORG) FILOCA 
                           ON (V.CDCAIXA = FILOCA.CDCAIXA
                            AND V.CDLOJA = FILOCA.CDLOJA
                            AND V.CDFILIAL = FILOCA.CDFILIAL
                            AND TO_CHAR(V.DTENTRVENDA, 'MM') = FILOCA.CDMES
                            AND TO_CHAR(V.DTENTRVENDA, 'YYYY') = FILOCA.CDANO
                            AND V.NRORG = FILOCA.NRORG)
        					WHERE (INSTR('MES_BAL_DLV_TAA_CMD_AJU_AUT', SUBSTR(V.IDORIGEMVENDA, 0, 3)) > 0 OR INSTR('MES_BAL_DLV_TAA_CMD_AJU_AUT', SUBSTR(V.IDORIGEMVENDA, 5, 7)) > 0)
              ) FILOCAVE
             ON FILOCAVE.CDFILIAL = I.CDFILIAL
            AND FILOCAVE.NRORG = I.NRORG
            AND FILOCAVE.NRSEQVENDA = I.NRSEQVENDA            
            AND FILOCAVE.CDCAIXA = I.CDCAIXA
            AND I.CDPRODUTO <> (CASE WHEN (FILOCAVE.IDTRATTAXAENTR = 'P' AND SUBSTR(FILOCAVE.IDORIGEMVENDA, 0, 3) = 'DLV') THEN FILOCAVE.CDPRODTAXAENTR
                                                   ELSE 'ZZZZZZZ'
                                               END)
            AND I.CDPRODUTO <> (CASE WHEN (FILOCAVE.IDTRATTAXASERV = 'P' AND FILOCAVE.IDCOMISVENDA IN ('S', 'G')) THEN FILOCAVE.CDPRODTAXASERV
                                                   ELSE 'ZZZZZZZ'
                                               END)
            AND INSTR(:MES, TO_CHAR(FILOCAVE.DTENTRVENDA, 'MM')) > 0
            AND TO_CHAR(FILOCAVE.DTENTRVENDA, 'YYYY') = :ANO
        WHERE FILOCAVE.IDSITUVENDA IN(CASE SUBSTR(FILOCAVE.IDORIGEMVENDA, 0, 3)
                                                   WHEN 'AJU' THEN FILOCAVE.IDSITUVENDA
                                                   ELSE 'O'
                                               END)
        GROUP BY FILOCAVE.CDFILIAL, FILOCAVE.CDANO, FILOCAVE.CDMES, FILOCAVE.NRORG, FILOCAVE.NMFILIAL, FILOCAVE.VRMES, TO_CHAR(DTENTRVENDA, 'MM'), TO_CHAR(DTENTRVENDA, 'YYYY'), FILOCAVE.NRPESMESAVENDA, FILOCAVE.IDSITUVENDA
        ORDER BY FILOCAVE.CDFILIAL, FILOCAVE.CDANO, FILOCAVE.CDMES)A
  ";
}